"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Get = void 0;
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = require("fs");
const file_1 = require("../../libs/file");
const format_1 = require("../../libs/format");
const logger_1 = require("../../libs/logger");
const progress_bar_1 = require("../../libs/progress-bar");
const stream_1 = require("stream");
const promises_1 = require("stream/promises");
const head_1 = require("./head");
class Get extends head_1.Head {
    method = 'get';
    responseType;
    saveTo;
    constructor({ responseType, saveTo, ...props }) {
        super(props);
        Object.assign(this, { responseType, saveTo });
    }
    async exec() {
        if (this.saveTo) {
            this.responseType = 'pipe';
            this.saveTo = this.scene.getPath(this.saveTo);
        }
        if (!this.responseType)
            this.responseType = 'json';
        return await super.exec();
    }
    async send(fetchOpts = {}) {
        const rs = await fetch(this.fullURLQuery, {
            method: this.method,
            headers: this.headers,
            ...fetchOpts
        });
        const data = await this.getResponseData(rs);
        this.response = {
            ok: rs.ok,
            headers: this.getResponseHeader(rs),
            data,
            status: rs.status,
            statusText: rs.statusText
        };
        return this.response;
    }
    async getResponseData(rs) {
        switch (this.responseType) {
            case 'none':
                return undefined;
            case 'json':
                return await rs.json();
            case 'blob':
                return await rs.blob();
            case 'buffer':
                return await rs.arrayBuffer();
            case 'pipe':
                if (!this.saveTo)
                    return undefined;
                const bar = this.logger.is(logger_1.LoggerLevel.INFO) ? new progress_bar_1.ProgressBar(this.logger.clone()) : undefined;
                if (this.title)
                    bar?.logger.addIndent();
                await bar?.init(chalk_1.default.gray.dim('Connecting to server...'));
                bar?.start();
                try {
                    const stream = (0, fs_1.createWriteStream)(this.saveTo, { autoClose: false, emitClose: false });
                    const body = stream_1.Readable.fromWeb(rs.body);
                    if (bar) {
                        let total = 0;
                        body.on('data', (chunk) => {
                            const len = chunk.byteLength;
                            total += len;
                            bar?.update(chalk_1.default.gray(`Downloading ${(0, format_1.formatNumber)(total / 1000, { maximumFractionDigits: 0 })} kbs | Rate: ${(0, format_1.formatNumber)(len, { maximumFractionDigits: 0 })} bytes`));
                        });
                    }
                    await (0, promises_1.finished)(body.pipe(stream));
                }
                finally {
                    if (this.title)
                        bar?.logger.addIndent(-1);
                    bar?.stop();
                }
                return new file_1.File(this.saveTo, this.scene);
            default:
                return await rs.text();
        }
    }
}
exports.Get = Get;
//# sourceMappingURL=get.js.map