"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const querystring_1 = require("querystring");
const file_store_1 = require("../../../components/file/file-store");
const file_temp_1 = require("../../../libs/file-temp");
const testing_1 = require("../../../testing");
const sub_1 = require("./sub");
let sub;
const tmp = new file_temp_1.FileTemp();
beforeEach(async () => {
    await testing_1.Testing.reset();
});
afterEach(async () => {
    await sub.dispose();
    tmp.remove();
});
test('Listen to handle jobs in queue', async () => {
    const jobData = { foo: 'bar' };
    sub = await testing_1.Testing.newElement(sub_1.Sub, {
        address: '0.0.0.0:4001',
        queue: {
            file: tmp.file
        },
        runs: [
            {
                vars: {
                    jobData: '${this.parentState.jobData}'
                }
            },
            {
                "http/jobs'stop": null
            }
        ]
    });
    setTimeout(async () => {
        await fetch('http://0.0.0.0:4001', {
            method: 'POST',
            body: JSON.stringify(jobData)
        });
    }, 1000);
    await sub.exec();
    expect(testing_1.Testing.vars.jobData).toEqual(jobData);
});
test('Check basic authentication via headers', async () => {
    const jobData = { foo: 'bar' };
    sub = await testing_1.Testing.newElement(sub_1.Sub, {
        address: '0.0.0.0:4001',
        secure: {
            basic: {
                username: 'thanh',
                password: '123'
            }
        },
        runs: [
            {
                vars: {
                    jobData: '${this.parentState.jobData}'
                }
            },
            {
                "http/jobs'stop": null
            }
        ]
    });
    setTimeout(async () => {
        const resp = await fetch('http://0.0.0.0:4001', {
            method: 'POST',
            headers: {
                authorization: `Basic ${testing_1.Testing.rootScene.globalUtils.base64.encrypt('thanh:123')}`
            },
            body: JSON.stringify(jobData)
        });
        expect(resp.status).toBe(201);
        expect(resp.statusText).toBe('secured');
    }, 1000);
    await sub.exec();
    expect(testing_1.Testing.vars.jobData).toEqual(jobData);
});
test('Check basic authentication via querystring', async () => {
    const jobData = { foo: 'bar' };
    sub = await testing_1.Testing.newElement(sub_1.Sub, {
        address: '0.0.0.0:4001',
        secure: {
            basic: {
                username: 'thanh',
                password: '123'
            }
        },
        runs: [
            {
                vars: {
                    jobData: '${this.parentState.jobData}'
                }
            },
            {
                "http/jobs'stop": null
            }
        ]
    });
    setTimeout(async () => {
        const token = (0, querystring_1.escape)(`Basic ${testing_1.Testing.rootScene.globalUtils.base64.encrypt('thanh:123')}`);
        const resp = await fetch(`http://0.0.0.0:4001?authorization=${token}`, {
            method: 'POST',
            body: JSON.stringify(jobData)
        });
        expect(resp.status).toBe(201);
        expect(resp.statusText).toBe('secured');
    }, 1000);
    await sub.exec();
    expect(testing_1.Testing.vars.jobData.foo).toEqual('bar');
});
test('Test storage is ref ID', async () => {
    const jobData = { foo: 'bar' };
    const fs = await testing_1.Testing.newElement(file_store_1.FileStore, {
        id: 'fileStorage',
        path: tmp.file,
        initData: []
    });
    await fs.exec();
    sub = await testing_1.Testing.newElement(sub_1.Sub, {
        address: '0.0.0.0:4002',
        queue: {
            storage: '${vars.fileStorage}'
        },
        runs: [
            {
                vars: {
                    jobData: '${this.parentState.jobData}',
                    jobInfo: '${this.parentState.jobInfo}'
                }
            },
            {
                "http/jobs'stop": null
            }
        ]
    });
    const t = new Promise((resolve, reject) => {
        setTimeout(async () => {
            try {
                await fetch('http://0.0.0.0:4002/test?foo=bar', {
                    method: 'POST',
                    headers: {
                        author: 'thanh'
                    },
                    body: JSON.stringify(jobData)
                });
            }
            catch (err) {
                reject(err);
            }
            resolve(undefined);
        }, 5000);
    });
    await sub.exec();
    await t;
    expect(testing_1.Testing.vars.jobData).toEqual(jobData);
    expect(testing_1.Testing.vars.jobInfo.path).toBe('/test');
    expect(testing_1.Testing.vars.jobInfo.query).toEqual({ foo: 'bar' });
    expect(testing_1.Testing.vars.jobInfo.method).toBe('POST');
    expect(testing_1.Testing.vars.jobInfo.headers.author).toBe('thanh');
    expect(tmp.isExisted).toBe(true);
    await fs.dispose();
});
//# sourceMappingURL=sub.spec.js.map