"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PackagesManager = void 0;
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const path_1 = require("path");
const exec_1 = require("./../components/exec/exec");
const logger_1 = require("./../libs/logger");
const progress_bar_1 = require("./../libs/progress-bar");
const modules_manager_1 = require("./modules-manager");
const PackageManagerSupported = ['yarn', 'npm'];
class PackagesManager {
    scene;
    bin;
    logger;
    tag = 'package-manager';
    get cmdInstall() {
        return this.bin === 'npm'
            ? ['npm', 'install', '--prefix', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`]
            : ['yarn', 'add', '--cwd', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
    }
    get cmdUninstall() {
        return this.bin === 'npm'
            ? ['npm', 'uninstall', '--prefix', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`]
            : ['yarn', 'remove', '--cwd', `${(0, path_1.join)(modules_manager_1.nodeModulesDir, '..')}`];
    }
    static get Dependencies() {
        const packageJSON = (0, fs_1.readFileSync)((0, path_1.join)(modules_manager_1.nodeModulesDir, '../package.json')).toString();
        return JSON.parse(packageJSON).dependencies || {};
    }
    constructor(scene) {
        this.scene = scene;
        this.logger = this.scene.logger.clone(this.tag);
        this.bin = PackageManagerSupported.find((bin) => {
            try {
                const log = (0, child_process_1.execSync)(`${bin} -v`).toString();
                return !!log;
            }
            catch { }
            return false;
        });
        if (!this.bin)
            throw new Error('Could not found "npm" or "yarn" to install lack packages');
    }
    async install(...packages) {
        if (!packages.length)
            return;
        const cmd = [...this.cmdInstall, ...packages];
        const msg = `tags ${packages.map(e => `"${e}"`).join(', ')}`;
        const bar = !this.logger.is(logger_1.LoggerLevel.SILENT) ? new progress_bar_1.ProgressBar(this.logger) : undefined;
        try {
            if (bar) {
                await bar.init(`Installing ${msg}`);
                bar.start();
            }
            else {
                this.logger.debug(`Installing ${msg}`);
            }
            const exec = this.scene.elementBuilder
                .element(exec_1.Exec, {
                commands: cmd
            })
                .tag(this.tag)
                .logger(this.logger)
                .build();
            await exec.exec();
            if (bar) {
                bar.passed(`Installed ${msg} successfully`);
            }
            else {
                this.logger.info(`Installed ${msg} successfully`);
            }
        }
        catch (err) {
            if (bar) {
                bar.failed(`Installed ${msg} failed`);
            }
            else {
                this.logger.error(`Installed ${msg} failed`);
            }
            throw err;
        }
    }
    async uninstall(...packages) {
        packages = packages.filter(name => !!this.isInstalled(name));
        if (!packages.length)
            return;
        const cmd = [...this.cmdUninstall, ...packages];
        const msg = `tags ${packages.map(e => `"${e}"`).join(', ')}`;
        const bar = !this.logger.is(logger_1.LoggerLevel.SILENT) ? new progress_bar_1.ProgressBar(this.logger) : undefined;
        try {
            if (bar) {
                await bar.init(`Uninstalling ${msg}`);
                bar.start();
            }
            else {
                this.logger.debug(`Uninstalling ${msg}`);
            }
            const exec = this.scene.elementBuilder
                .element(exec_1.Exec, {
                commands: cmd
            })
                .tag(this.tag)
                .logger(this.logger)
                .build();
            await exec.exec();
            if (bar) {
                bar.passed(`Uninstalled ${msg} successfully`);
            }
            else {
                this.logger.info(`Uninstalled ${msg} successfully`);
            }
        }
        catch (err) {
            if (bar) {
                bar.failed(`Uninstalled ${msg} failed`);
            }
            else {
                this.logger.error(`Uninstalled ${msg} failed`);
            }
            throw err;
        }
    }
    async clean() {
        const modules = Object.keys(PackagesManager.Dependencies);
        await this.uninstall(...modules);
        return modules;
    }
    isInstalled(...packages) {
        return packages.map(packageName => {
            const path = (0, path_1.join)(modules_manager_1.nodeModulesDir, packageName);
            return (0, fs_1.existsSync)(path) ? path : undefined;
        });
    }
}
exports.PackagesManager = PackagesManager;
//# sourceMappingURL=packages-manager.js.map